import random
from dataclasses import dataclass

from lanoel.utils import (
    has_duplicate_names,
    inner_category,
    max_count,
    outter_category,
)


@dataclass
class Participant:
    name: str
    category: str


@dataclass
class Pair:
    giver: str
    receiver: str


def secret_santa(participants: list[Participant]):

    if has_duplicate_names(participants):
        raise Exception(
            "Their are a duplicate name in the participants list.",
        )

    givers = participants.copy()
    receivers = participants.copy()

    pair = []
    while True:
        category = max_count(givers)

        inner = inner_category(givers, category)
        outter = outter_category(receivers, category)

        giver = random.choice(inner)
        receiver = random.choice(outter if len(outter) > 0 else participants)

        givers.remove(giver)
        receivers.remove(receiver)

        pair.append(Pair(giver.name, receiver.name))

        if len(givers) == 0:
            break

    return pair


if __name__ == "__main__":
    participants = [Participant("Oliver", "Tuner")]
    res = secret_santa(participants)
    print(res)



def max_count(data):
    # Get the key with the maximum list length
    count = {}
    for entry in data:
        cat = entry.category
        if cat in count:
            count[cat] += 1
        else:
            count[cat] = 0

    return max(count, key=count.get)


def inner_category(data, category):
    return [entry for entry in data if entry.category == category]


def outter_category(data, category):
    return [entry for entry in data if entry.category != category]


def has_duplicate_names(data):
    name_set = set()
    for entry in data:
        name = entry.name
        if name in name_set:
            return True
        name_set.add(name)
    return False



from argparse import ArgumentParser
from lanoel.lanoel import secret_santa, Participant, Pair
import pathlib
import sys

DEFAULT_CATEGORY = "default"
DEFAULT_SEPARATOR = ","
DEFAULT_CSV_SEPARATOR = ";"


def parse_participant(participant_str: str, separator: str) -> Participant:
    """Parse a participant string and return a Participant object."""

    if separator not in participant_str:
        return Participant(participant_str.strip(), DEFAULT_CATEGORY)

    participant_args = participant_str.split(separator)
    if len(participant_args) != 2:
        sys.exit(f"Invalid participant string: {participant_str}")
    name, category = participant_args

    return Participant(name.strip(), category.strip())


def output_participants(participants: list[Participant]):
    print("-----Participants-----")
    for participant in participants:
        print(f"{participant.name} ({participant.category})")
    print()


def output_pairs(result: list[Pair]):
    print("-----Pairs-----")
    for pair in result:
        print(f"{pair.giver} est le secret santa de {pair.receiver}.")
    print()


def entrypoint():
    parser = ArgumentParser()

    parser.add_argument(
        "-p",
        "--participant",
        type=str,
        action="append",
        help=f"The name of participants and optionally category separated by a semicolon. If category is not provided, the participant will be in the default category. ex: Oliver{DEFAULT_SEPARATOR}Tuner",
    )
    parser.add_argument(
        "-f",
        "--file",
        type=pathlib.Path,
        help=f"The path of the csv file. The file should be in the format 'name{DEFAULT_CSV_SEPARATOR}category'.",
    )

    args = parser.parse_args()
    print(args)

    participants = []
    if args.participant:
        participants.extend(
            [parse_participant(p, DEFAULT_SEPARATOR) for p in args.participant]
        )

    if args.file:
        with open(args.file, "r", encoding="utf-8") as participants_file:
            participants_lines = participants_file.readlines()
            participants.extend(
                [
                    parse_participant(p, DEFAULT_CSV_SEPARATOR)
                    for p in participants_lines
                ]
            )

    if len(participants) <= 1:
        sys.exit("The number of participants must be greater than 1.")

    output_participants(participants)

    result = secret_santa(participants)

    if len(result) != len(participants):
        sys.exit("The number of participants does not match the number of pairs.")

    output_pairs(result)



#     category_field = "cat"
#     json_path = "members.json"

#     with open(json_path) as members_file:
#         members = json.load(members_file)

#     pair = secret_santa(members)

#     emoji = "🔔🎉🎅🎁🎄🧦🌟"
#     for m in pair:
#         print(f"{random.choice(emoji)} {m[0]['name'].capitalize()} est le secret santa de {m[1]['name'].capitalize()}.")
